//------------------------------------------------------------------------
// NRF 905 Hardware Driver
//
// Remo Keller
// Ascom Systec AG
// 7th May 2004
//------------------------------------------------------------------------

#include <msp430x14x.h>
#include "physical.h"
#include "defs.h"
#include "hwinit.h"
#include "timer_a.h"

//------------------------------------------------------------------------
//  NRF global variables
//------------------------------------------------------------------------
t_NRF_Buffer         uNrfBuffer;
t_NRF_StatReg        uNrfStatReg = { 0, 0 };  // status register
t_NRF_Config_Reg     uNrfConfigReg = {       // configuration struct
     NRF_CH_0_INIT,         // init Channel Number
     NRF_HFREQ_PLL_INIT,    // init PLL Frequency
     NRF_PA_PWR_INIT,       // PA Output Power
     NRF_RX_RED_PWR_INIT,   // Rx Power
     NRF_AUTO_RET_INIT,     // Retransmit enable
     NRF_RX_AFW_INIT,       // Rx Address witdh
     NRF_TX_AFW_INIT,       // Tx Address witdh
     NRF_RX_PW_INIT,        // Rx Payload witdh
     NRF_TX_PW_INIT,        // Tx Payload witdh
     ( unsigned char )( NRF_RX_ADDRESS_INIT ),                // Rx Address Byte 0
     ( unsigned char )( NRF_RX_ADDRESS_INIT >> SHIFTBYTE_1 ), // Rx Address Byte 1
     ( unsigned char )( NRF_RX_ADDRESS_INIT >> SHIFTBYTE_2 ), // Rx Address Byte 2
     ( unsigned char )( NRF_RX_ADDRESS_INIT >> SHIFTBYTE_3 ), // Rx Address Byte 3
     NRF_UP_CLK_FREQ_INIT,  // Output clock frequency
     NRF_UP_CLK_EN_INIT,    // Output clock enable
     NRF_XOF_INIT,          // Crystal oscillator frequency
     NRF_CRC_EN_INIT,       // CRC enable
     NRF_CRC_MODE_INIT      // CRC mode
};

t_NRF_Channel        asNrfChannel[ NRF_MAX_CH_NUMBER ] ={
    NRF_CH_0_INIT, 0,           
    NRF_CH_1_INIT, 0,                  
    NRF_CH_2_INIT, 0,                  
    NRF_CH_3_INIT, 0,       
    NRF_CH_4_INIT, 0,      
    NRF_CH_5_INIT, 0,      
    NRF_CH_6_INIT, 0,      
    NRF_CH_7_INIT, 0,      
    NRF_CH_8_INIT, 0,      
    NRF_CH_9_INIT, 0,      
    NRF_CH_10_INIT, 0,      
    NRF_CH_11_INIT, 0,    
    NRF_CH_12_INIT, 0,     
    NRF_CH_13_INIT, 0,     
    NRF_CH_14_INIT, 0,     
    NRF_CH_15_INIT, 0     
};


//------------------------------------------------------------------------
//  SPI variables
//------------------------------------------------------------------------
//--- local variables ( static )

//------------------------------------------------------------------------
// local prototypes
//------------------------------------------------------------------------
void SPI_ReadSpi( unsigned char ucUpLoadType );
void SPI_WriteSpi( unsigned char ucDwnLoadType );
void SPI_Enable( void );
void SPI_Disable( void );

//------------------------------------------------------------------------
//  NRF functions
//------------------------------------------------------------------------
void NRF_SetPwrMode( unsigned char ucPowerMode )
{
    switch( ucPowerMode ){
        case POWER_OFF:     // rf supply disabled
            SET_BITS( P2OUT, P3V0RF_EN );
            CLEAR_BITS( P5OUT, NRF_PWRUP );
            CLEAR_BITS( P2OUT, NRF_TRX_CE );
        break;
        case SLEEP:         // rf supply enabled, nrf powered down
            CLEAR_BITS( P2OUT, P3V0RF_EN );
            CLEAR_BITS( P5OUT, NRF_PWRUP );
            CLEAR_BITS( P2OUT, NRF_TRX_CE );
        break;
        case STANDBY:       // nrf powered up, no tx or rx operation
            CLEAR_BITS( P2OUT, P3V0RF_EN );
            CLEAR_BITS( P2OUT, NRF_TRX_CE );
            SET_BITS( P5OUT, NRF_PWRUP );
        break;
        case REDUCED_RX:    // nrf in reduced rx mode
            CLEAR_BITS( P2OUT, P3V0RF_EN );
            SET_BITS( P2OUT, NRF_TRX_CE );
            SET_BITS( P5OUT, NRF_PWRUP );
            // SET REDUCED RX POWER MODE OVER SPI
        break;
        case FULL_POWER:    // nrf in full operation
            CLEAR_BITS( P2OUT, P3V0RF_EN );
            SET_BITS( P2OUT, NRF_TRX_CE );
            SET_BITS( P5OUT, NRF_PWRUP );
            // CLEAR REDUCED RX POWER MODE OVER SPI
        break;
    }
}
//------------------------------------------------------------------------
void NRF_Init( void )
{
    // full initialization 
    SPI_WriteSpi( WRITE_CONFIG );
}
//------------------------------------------------------------------------
void NRF_SetChannel( unsigned int uiChannel )
{
    uNrfConfigReg.uNrfConfigBf.Nrf905ChNo = uiChannel;
    SPI_WriteSpi( WRITE_CHANNEL_CONFIG );
}
//------------------------------------------------------------------------
void NRF_SetTxPwr( unsigned char uiTxPwr )
{
    uNrfConfigReg.uNrfConfigBf.Nrf905PaPwr = uiTxPwr;
    SPI_WriteSpi( WRITE_CHANNEL_CONFIG );
}
//------------------------------------------------------------------------
void NRF_SetRxAddress( unsigned long ulRxAddress )
{
    uNrfConfigReg.uNrfConfigBf.Nrf905RxAddress_0 = ( unsigned char )( ulRxAddress );
    uNrfConfigReg.uNrfConfigBf.Nrf905RxAddress_1 = ( unsigned char )( ulRxAddress >> SHIFTBYTE_1 ); // Rx Address Byte 1
    uNrfConfigReg.uNrfConfigBf.Nrf905RxAddress_2 = ( unsigned char )( ulRxAddress >> SHIFTBYTE_2 ); // Rx Address Byte 2
    uNrfConfigReg.uNrfConfigBf.Nrf905RxAddress_3 = ( unsigned char )( ulRxAddress >> SHIFTBYTE_3 ); // Rx Address Byte 3
    SPI_WriteSpi( WRITE_RX_ADDRESS );
}
//------------------------------------------------------------------------
void NRF_SetTxAddress( unsigned long ulTxAddress )
{
    uNrfBuffer.TxAddress[0] = ( unsigned char )( ulTxAddress );
    uNrfBuffer.TxAddress[1] = ( unsigned char )( ulTxAddress >> SHIFTBYTE_1 ); // Tx Address Byte 1
    uNrfBuffer.TxAddress[2] = ( unsigned char )( ulTxAddress >> SHIFTBYTE_2 ); // Tx Address Byte 2
    uNrfBuffer.TxAddress[3] = ( unsigned char )( ulTxAddress >> SHIFTBYTE_3 ); // Tx Address Byte 3
    SPI_WriteSpi( WRITE_TX_ADDRESS );
}
//------------------------------------------------------------------------
void NRF_GetTxAddress( void )
{
    SPI_ReadSpi( READ_TX_ADDRESS );
}
//------------------------------------------------------------------------
void NRF_ReadConfig( void )
{
    SPI_ReadSpi( READ_CONFIG );
}
//------------------------------------------------------------------------
void NRF_TxDownLoad( char* pTxStream, unsigned char ucStreamLength )
{


}
//------------------------------------------------------------------------
unsigned char NRF_RxUpLoad( char*pDestAddress )
{

    return( 1 );    // return length of uploaded datas
}
//------------------------------------------------------------------------
void NRF_Send( void )
{
    // enable transmission
}

//------------------------------------------------------------------------
t_NRF_Channel* getNrfChannel( void )
{
    return( ( t_NRF_Channel* )&asNrfChannel );
} 

//------------------------------------------------------------------------
unsigned char NRF_CarrierSense( unsigned char ucChannelNumber, unsigned long ulScanTime )
{
    unsigned long *pulScanTime;
    unsigned char ucCarDetLevel;  // actual carrier detect level 0 or 1
    unsigned long ulDutyCycle;
    

    NRF_SetPwrMode( STANDBY );
    NRF_SetChannel( ucChannelNumber );
    NRF_SetPwrMode( FULL_POWER );
    SET_BITS( P2OUT, NRF_TRX_CE );                  // set receiver mode
    
    ClearTimeMeasure( );                            // clear time measure watch
    ucCarDetLevel = P2IN & NRF_CD;                  // save carrier detect level
    if( ucCarDetLevel )
        StartTimeMeasure( );
    pulScanTime = SetTimeOut( ulScanTime );
    while( *pulScanTime ){
        if( ( P2IN & NRF_CD ) != ucCarDetLevel ){   // carrier detect level has changed
            ucCarDetLevel = P2IN & NRF_CD;          // save carrier detect level
            if( ucCarDetLevel )                     // carrier sense goes high
                StartTimeMeasure( );                // begin to measure high time
            else
                StopTimeMeasure( );
        }            
    }    
    StopTimeMeasure( );
    ulDutyCycle =  *ReadOutTimeMeasure( ) * 100;
    ulDutyCycle = (unsigned char)( ulDutyCycle / ulScanTime );
    CLEAR_BITS( P2OUT, NRF_TRX_CE );                // clear receiver mode
    return( (unsigned char) ulDutyCycle );
}

//------------------------------------------------------------------------
//  SPI functions
//------------------------------------------------------------------------
//--- enable spi ---------------------------------------------------------
void SPI_Enable( void )
{
    SET_BITS( P5OUT, NRF_CSN );		  		    // set csn pin high
    SET_BITS( P5DIR, NRF_CSN );		         	    // set csn pin as output
    SET_BITS( P5SEL, SPI1_SIMO + SPI1_MOSI + SPI1_UCLK );   // P5.1,2,3 SPI option select
    U1CTL = CHAR + SYNC + MM + SWRST;			    // 8-bit, SPI, Master
    U1TCTL = CKPH + SSEL1 + STC;			    // Polarity, SMCLK, 3-wire
    U1BR0 = 0x02;	    				    // SPICLK = SMCLK/2
    U1BR1 = 0x00;                     
    U1MCTL = 0x00; 
    SET_BITS( ME2, USPIE1 );	    // Module enable
    CLEAR_BITS( U1CTL, SWRST );	    // SPI enable
    CLEAR_BITS( IFG2, URXIFG1 );    // clear Rx interrupt flag
    CLEAR_BITS( P5OUT, NRF_CSN );   // set csn pin low -> enable spi 

}
//--- disable spi --------------------------------------------------------
void SPI_Disable( void )
{
    CLEAR_BITS( IE2, URXIE1 );      // RX interrupt disable
    SET_BITS( P5OUT, NRF_CSN );     // set csn pin high -> disable spi 
}
//--- read to spi --------------------------------------------------------
void SPI_ReadSpi( unsigned char ucUpLoadType )
{
    unsigned char  ucSpiInstr;
    unsigned char  ucUpLoadLength;
    unsigned char* pucUpLoadStream;
    
    switch( ucUpLoadType ){
        case READ_CONFIG:
            ucSpiInstr = NRF_SPI_INSTR_R_CONFIG;
            ucUpLoadLength = sizeof( uNrfConfigReg );
            pucUpLoadStream = (unsigned char*)( &uNrfConfigReg );
        break;  
        case READ_TX_ADDRESS:
            ucSpiInstr = NRF_SPI_INSTR_R_TX_ADDRESS;
            ucUpLoadLength = TX_ADDRESS_SIZE;
            pucUpLoadStream = (unsigned char*)( &uNrfBuffer.TxAddress[0] );
        break;  
    }
    SPI_Enable( );
    while (!( IFG2 & UTXIFG1 ) );           // USART1 TX buffer ready?
    TXBUF1 = ucSpiInstr;                    // write spi instructio word
    while( !( IFG2 & URXIFG1 ) );           // wait for rx interrupt
    uNrfStatReg.ucNrfStatReg = RXBUF1;      // first received byte = status byte
    
    while( ucUpLoadLength ){
        ucUpLoadLength--;                  // decrement stream length
        while (!( IFG2 & UTXIFG1 ) );      // USART1 TX buffer ready?
        TXBUF1 = 0x00;                     // send dummy write instruction
        while( !( IFG2 & URXIFG1 ) );          // wait for rx buffer
        *pucUpLoadStream = RXBUF1;
        pucUpLoadStream++;
    }
    SPI_Disable( );
    
}
//--- write to spi -------------------------------------------------------
void SPI_WriteSpi( unsigned char ucDwnLoadType )
{
    unsigned char ucSpiInstr;
    unsigned char ucDownLoadLength;
    unsigned char* pucDownLoadStream;
    
    switch( ucDwnLoadType ){
        case WRITE_RX_ADDRESS:    
            ucSpiInstr = NRF_SPI_INSTR_W_CONFIG + LOC_RX_ADDRESS;
            ucDownLoadLength = RX_ADDRESS_SIZE + 1;
            pucDownLoadStream = &uNrfConfigReg.aucNrfConfig[LOC_RX_ADDRESS];
        break;
        case WRITE_TX_ADDRESS:
            ucSpiInstr = NRF_SPI_INSTR_W_TX_ADDRESS;
            ucDownLoadLength = TX_ADDRESS_SIZE;
            pucDownLoadStream = &uNrfBuffer.TxAddress[0];
        break;
        case WRITE_CHANNEL_CONFIG:
            ucSpiInstr = NRF_SPI_INSTR_CH_CONFIG;
            ucSpiInstr += uNrfConfigReg.uNrfConfigBf.Nrf905ChNo >> 8 ;
            ucSpiInstr += uNrfConfigReg.uNrfConfigBf.Nrf905HfreqPll << 1;
            ucSpiInstr += uNrfConfigReg.uNrfConfigBf.Nrf905PaPwr << 2;
            ucDownLoadLength = 1;
            pucDownLoadStream = &uNrfConfigReg.aucNrfConfig[LOC_CHANNEL];
        break;
        case WRITE_TX_PAYLOAD:
            ucSpiInstr = NRF_SPI_INSTR_W_TX_PAYLOAD;

        break;
        case WRITE_CONFIG:
            ucSpiInstr = NRF_SPI_INSTR_W_CONFIG;
            ucDownLoadLength = sizeof( uNrfConfigReg );
            pucDownLoadStream = (unsigned char*)&uNrfConfigReg;
        break;
    }
    SPI_Enable( );
    while (!( IFG2 & UTXIFG1 ) );           // USART1 TX buffer ready?
    TXBUF1 = ucSpiInstr;                    // write spi instructio word

    while( ucDownLoadLength ){              // Spi stream not empty?
        ucDownLoadLength--;                 // decrement spi length
        while (!( IFG2 & UTXIFG1 ) );       // USART1 TX buffer ready?
        TXBUF1 = *pucDownLoadStream;        // load spi tx buffer
        pucDownLoadStream++;                // increment pointer to spi stream
    }
    while (!( IFG2 & UTXIFG1 ) );       // USART1 TX buffer ready?
    SPI_Disable( );
}

//------------------------------------------------------------------------
// END OF FILE
//------------------------------------------------------------------------


